package com.someluigi.tuc.server;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.util.Map;

import org.simpleframework.http.Address;
import org.simpleframework.http.Request;
import org.simpleframework.http.Response;
import org.simpleframework.http.core.Container;
import org.simpleframework.http.core.ContainerServer;
import org.simpleframework.transport.Server;
import org.simpleframework.transport.connect.Connection;
import org.simpleframework.transport.connect.SocketConnection;

import com.someluigi.tuc.server.registry.BlitemResponder;
import com.someluigi.tuc.server.registry.BlitemStack;
import com.someluigi.tuc.server.registry.BlitemType;
import com.someluigi.tuc.server.registry.GameRegistry;
import com.someluigi.tuc.server.registry.Recipe;
import com.someluigi.tuc.server.util.FileUtils;

public class HttpServer implements Container {
	
	public static Connection connection;
	public PrintStream stdOut = System.out;

	public static void start() throws Exception {
		Container container = new HttpServer();
		Server server = new ContainerServer(container);
		connection = new SocketConnection(server);
		SocketAddress address = new InetSocketAddress(Integer.valueOf(StartMain.args[0]));
		
		connection.connect(address);
		
		//connection.close();
	}
	
	public static void stop() throws IOException {
		connection.close();
	}

	@Override
	public void handle(Request req, Response res) {
		try {
			PrintStream body = res.getPrintStream();
			long time = System.currentTimeMillis();
			   
			res.setValue("Server", "TUCServer/"+StartMain.version+" (theuchain)");
			res.setDate("Date", time);
			res.setDate("Last-Modified", time);
			
			
			Address a = req.getAddress();
			
			// notes: domain is null
			
			
			String[] seg = a.getPath().getSegments();
			if (seg.length>0) {
				if (seg[0].equalsIgnoreCase("res") || seg[0].equalsIgnoreCase("html")) {
					
					File rf = new File("../" + a.getPath().getPath(0));
					
					if (! rf.exists()) {
						res.setCode(404);
						res.setValue("Content-Type", "text/html");
						body.println("<h2>404</h2><br>The resource was not available. This is not intended to be user-browsed. ");
						body.close();
						return;
					}
					
					String ext = a.getPath().getExtension();
					if (ext.equalsIgnoreCase("png")) {
						res.setValue("Content-Type", "image/png");
					} else if (ext.equalsIgnoreCase("gif")) {
						res.setValue("Content-Type", "image/gif");
					} else if (ext.equalsIgnoreCase("jml")) {
						res.setValue("Content-Type", "text/html");
						String jml = FileUtils.readFile("../" + a.getPath().getPath(0), "UTF-8");
						if (jml.contains("%%%mode=DEBUG%%%")) {
							res.setCode(405);
							body.println("<h2>405 Method Not Allowed</h2><br>The method is not allowed because the server does not have debug mode enabled.");
							body.close();
							return;
						}
					} else {
						res.setCode(500);
						res.setValue("Content-Type", "text/html");
						body.println("<h2>500 Internal TUC Server Error</h2><br>The resource does not have a known MIME-Type, cannot transfer properly. Extension: " + ext);
						body.close();
						return;
					}
					
					// a resource was called!
					// the resources are usually binary files, so we can't copy them
					// as text
					
					// but we will do this with text files anyway, for simplicity
					
					// Get a Stream from the file
					FileInputStream reader = new FileInputStream(rf);
					
			        byte[] buffer = new byte[153600];
			        int bytesRead = 0;
			 
			        while ((bytesRead = reader.read(buffer)) > 0)
			        {  
			        	// copy the bytes from the filein stream to the httpout stream
			        	body.write(buffer, 0, bytesRead);
			        	buffer = new byte[153600];
			        }
			        
			        body.close(); // Close Stream
			        reader.close(); // Close Stream
			        return; // Exit from HTTP Processing
				}
			}
			
			if (a.toString().equalsIgnoreCase("/stop")) {
				body.println("Stopping Server");
				body.close();
				HttpServer.stop();
			}
			
			if (a.toString().equalsIgnoreCase("/registry")) {
				
				res.setValue("Content-Type", "text/html"); // we're working in HTML
				// now baby!
				
				body.println("<h1>TUC Game Registry (com.someluigi.tuc.server.registry.GameRegistry)");
				body.println("<br><h3>Item Registry</h3>");
				
				for (Map.Entry<Integer, BlitemResponder> entry : GameRegistry.blitems.entrySet()) {
					Integer key = entry.getKey();
				    BlitemResponder c = entry.getValue();
				    if (c.getBlitemType().equals(BlitemType.ITEM)) {
				    	body.println("<br>I: " + key + " = " + c);
				    }
				}
				
				body.println("<br><h3>Block Registry</h3>");
				
				for (Map.Entry<Integer, BlitemResponder> entry : GameRegistry.blitems.entrySet()) {
					Integer key = entry.getKey();
				    BlitemResponder c = entry.getValue();
				    if (c.getBlitemType().equals(BlitemType.BLOCK)) {
				    	body.println("<br>B: " + key + " = " + c);
				    }
				}
				
				body.println("<br><h3>Recipe Registry</h3>");
								
				for (Recipe r: GameRegistry.recipes) {
					
					
					body.println("<hr>Recipe for:");
					for (BlitemStack out: r.result) {
						body.println("<br>" + out);
					}
					body.println("<br><table>");
					for(int y = 1; y <= r.h; ++y) {
						body.println("<tr>");
						for(int x = 1; x <= r.w; ++x) {
							body.println("<td>");
							body.println("<br>" + x + " " + y + " " + r.oa[(((y-1)*r.w) + x)-1]);
							body.println("</td>");
						}
						body.println("</tr>");
					}
					body.println("</table>");
				}
				
				body.close(); // close the body early
				return; // stop processing
			}
			
			body.println("Accessing URL: " + req.getAddress());
			body.close();
	      } catch(Exception e) {
	    	  e.printStackTrace();
	      }
	}

}
