package com.someluigi.tuc.server.init;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import org.json.JSONObject;
import org.json.JSONTokener;

import com.someluigi.tuc.server.exception.ExitServerException;
import com.someluigi.tuc.server.util.SysStr;

public class ReadConfig {
	
	public enum AuthMethod {
		DEBUG_ANY,
		JSONFILE_LOGIN;
	} // don't need anything fancy here, just a SIMPLE enum
	
	public JSONObject json;
	
	public AuthMethod auth;
	public String authDir;
	
	public JSONObject apiControl; 
	
	public ReadConfig(String conffile) throws FileNotFoundException, ExitServerException {
		
		FileInputStream f = new FileInputStream(conffile);
		JSONTokener jt = new JSONTokener(f);
		json = new JSONObject(jt);
		
		String authm = json.optString("AuthenticationMethod", "<empty value>").toUpperCase();
		if (authm.equalsIgnoreCase("json-files")) {
			SysStr.stdOut.println("Running in JSON authentication using token secrets");
			if (json.has("AuthenticationDirectory")) {
				File d = new File(json.getString("AuthenticationDirectory"));
				if (! d.exists()) {
					SysStr.stdErr.println("CONFIGURATION ERROR:");
					SysStr.stdErr.println("AuthenticationDirectory is defined, but the directory it points to does not exist.");
					SysStr.stdErr.println("Nonexistant Directory: " + d.getAbsolutePath());
					throw new ExitServerException();
				}
				if (! d.isDirectory()) {
					SysStr.stdErr.println("CONFIGURATION ERROR:");
					SysStr.stdErr.println("AuthenticationDirectory is defined, and it exists, but it doesn't point to a directory.");
					SysStr.stdErr.println("Not a directory, please make it one: " + d.getAbsolutePath());
					throw new ExitServerException();
				}
				auth = AuthMethod.JSONFILE_LOGIN;
				authDir = json.getString("AuthenticationDirectory");
			} else {
				SysStr.stdErr.println("CONFIGURATION ERROR:");
				SysStr.stdErr.println("AuthenticationDirectory is undefined, and you are using JSON-Token Authentication.");
				SysStr.stdErr.println("Please set to a valid directory or use debug-mode authentication!");
				throw new ExitServerException();
			}
		} else if (authm.equalsIgnoreCase("debug-mode")) {
			auth = AuthMethod.DEBUG_ANY;
			SysStr.stdOut.println("WARNING: Running in debug mode. Anybody can use /name and freely select a name.");
		} else {
			SysStr.stdErr.println("CONFIGURATION ERROR:");
			SysStr.stdErr.println("AuthenticationMethod is misconfigured.");
			SysStr.stdErr.println(authm + " is not a valid AuthenticationMethod value!");
			throw new ExitServerException();
		}
		
		
	}
	
}
